package sun.util.locale.provider;

import java.util.Collections;
import java.util.spi.CalendarDataProvider;
import java.util.spi.TimeZoneNameProvider;
import java.util.spi.LocaleNameProvider;
import java.util.spi.CurrencyNameProvider;
import java.text.spi.NumberFormatProvider;
import java.text.spi.DecimalFormatSymbolsProvider;
import java.text.spi.DateFormatSymbolsProvider;
import java.text.spi.DateFormatProvider;
import java.text.spi.CollatorProvider;
import java.text.spi.BreakIteratorProvider;
import java.util.IllformedLocaleException;
import java.util.ResourceBundle;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import sun.util.logging.PlatformLogger;
import java.util.Iterator;
import java.util.concurrent.ConcurrentHashMap;
import java.util.Set;
import java.util.List;
import java.util.Locale;
import java.util.spi.LocaleServiceProvider;
import java.util.concurrent.ConcurrentMap;

public final class LocaleServiceProviderPool
{
    private static ConcurrentMap<Class<? extends LocaleServiceProvider>, LocaleServiceProviderPool> poolOfPools;
    private ConcurrentMap<LocaleProviderAdapter.Type, LocaleServiceProvider> providers;
    private ConcurrentMap<Locale, List<LocaleProviderAdapter.Type>> providersCache;
    private Set<Locale> availableLocales;
    private Class<? extends LocaleServiceProvider> providerClass;
    static final Class<LocaleServiceProvider>[] spiClasses;
    private static List<LocaleProviderAdapter.Type> NULL_LIST;
    
    public static LocaleServiceProviderPool getPool(Class<? extends LocaleServiceProvider> clazz) {
        LocaleServiceProviderPool localeServiceProviderPool = (LocaleServiceProviderPool)LocaleServiceProviderPool.poolOfPools.get(clazz);
        if (localeServiceProviderPool == null) {
            LocaleServiceProviderPool localeServiceProviderPool2 = new LocaleServiceProviderPool(clazz);
            localeServiceProviderPool = (LocaleServiceProviderPool)LocaleServiceProviderPool.poolOfPools.putIfAbsent(clazz, localeServiceProviderPool2);
            if (localeServiceProviderPool == null) {
                localeServiceProviderPool = localeServiceProviderPool2;
            }
        }
        return localeServiceProviderPool;
    }
    
    private LocaleServiceProviderPool(Class<? extends LocaleServiceProvider> providerClass) {
        this.providers = new ConcurrentHashMap();
        this.providersCache = new ConcurrentHashMap();
        this.availableLocales = null;
        this.providerClass = providerClass;
        for (LocaleProviderAdapter.Type type : LocaleProviderAdapter.getAdapterPreference()) {
            LocaleProviderAdapter forType = LocaleProviderAdapter.forType(type);
            if (forType != null) {
                LocaleServiceProvider localeServiceProvider = forType.getLocaleServiceProvider(providerClass);
                if (localeServiceProvider != null) {
                    this.providers.putIfAbsent(type, localeServiceProvider);
                }
                else {
                    continue;
                }
            }
            else {
                continue;
            }
        }
    }
    
    static void config(Class<?> clazz, String msg) {
        PlatformLogger.getLogger(clazz.getCanonicalName()).config(msg);
    }
    
    public static Locale[] getAllAvailableLocales() {
        return (Locale[])LocaleServiceProviderPool.AllAvailableLocales.allAvailableLocales.clone();
    }
    
    public Locale[] getAvailableLocales() {
        HashSet<Object> set = new HashSet();
        set.addAll(this.getAvailableLocaleSet());
        set.addAll(Arrays.asList(LocaleProviderAdapter.forJRE().getAvailableLocales()));
        Locale[] array = new Locale[set.size()];
        set.toArray(array);
        return array;
    }
    
    private synchronized Set<Locale> getAvailableLocaleSet() {
        if (this.availableLocales == null) {
            this.availableLocales = new HashSet();
            Iterator<Object> iterator = this.providers.values().iterator();
            while (iterator.hasNext()) {
                Locale[] availableLocales = ((LocaleServiceProvider)iterator.next()).getAvailableLocales();
                for (int length = availableLocales.length, i = 0; i < length; ++i) {
                    this.availableLocales.add(getLookupLocale(availableLocales[i]));
                }
            }
        }
        return this.availableLocales;
    }
    
    boolean hasProviders() {
        return this.providers.size() != 1 || (this.providers.get(LocaleProviderAdapter.Type.JRE) == null && this.providers.get(LocaleProviderAdapter.Type.FALLBACK) == null);
    }
    
    public <P extends LocaleServiceProvider, S> S getLocalizedObject(LocaleServiceProviderPool.LocalizedObjectGetter<P, S> localizedObjectGetter, Locale locale, Object... array) {
        return this.getLocalizedObjectImpl(localizedObjectGetter, locale, true, null, array);
    }
    
    public <P extends LocaleServiceProvider, S> S getLocalizedObject(LocaleServiceProviderPool.LocalizedObjectGetter<P, S> localizedObjectGetter, Locale locale, String s, Object... array) {
        return this.getLocalizedObjectImpl(localizedObjectGetter, locale, false, s, array);
    }
    
    private <P extends LocaleServiceProvider, S> S getLocalizedObjectImpl(LocaleServiceProviderPool.LocalizedObjectGetter<P, S> localizedObjectGetter, Locale obj, boolean b, String s, Object... array) {
        if (obj == null) {
            throw new NullPointerException();
        }
        if (!this.hasProviders()) {
            return localizedObjectGetter.getObject((P)this.providers.get(LocaleProviderAdapter.defaultLocaleProviderAdapter), obj, s, array);
        }
        List<Locale> lookupLocales = getLookupLocales(obj);
        Set<Locale> availableLocaleSet = this.getAvailableLocaleSet();
        for (Locale locale : lookupLocales) {
            if (availableLocaleSet.contains(locale)) {
                Iterator<LocaleProviderAdapter.Type> iterator2 = this.findProviders(locale).iterator();
                while (iterator2.hasNext()) {
                    LocaleServiceProvider obj2 = (P)this.providers.get((Object)iterator2.next());
                    Object object = localizedObjectGetter.getObject(obj2, obj, s, array);
                    if (object != null) {
                        return object;
                    }
                    if (b) {
                        config(LocaleServiceProviderPool.class, "A locale sensitive service provider returned null for a localized objects,  which should not happen.  provider: " + obj2 + " locale: " + obj);
                    }
                    else {
                        continue;
                    }
                }
            }
            else {
                continue;
            }
        }
        return null;
    }
    
    private List<LocaleProviderAdapter.Type> findProviders(Locale locale) {
        List<LocaleProviderAdapter.Type> null_LIST = (List)this.providersCache.get(locale);
        if (null_LIST == null) {
            for (LocaleProviderAdapter.Type type : LocaleProviderAdapter.getAdapterPreference()) {
                LocaleServiceProvider localeServiceProvider = (LocaleServiceProvider)this.providers.get(type);
                if (localeServiceProvider != null && localeServiceProvider.isSupportedLocale(locale)) {
                    if (null_LIST == null) {
                        null_LIST = new ArrayList(2);
                    }
                    null_LIST.add(type);
                }
                else {
                    continue;
                }
            }
            if (null_LIST == null) {
                null_LIST = LocaleServiceProviderPool.NULL_LIST;
            }
            List<Object> list = (List)this.providersCache.putIfAbsent(locale, null_LIST);
            if (list != null) {
                null_LIST = list;
            }
        }
        return null_LIST;
    }
    
    static List<Locale> getLookupLocales(Locale locale) {
        return ResourceBundle.Control.getNoFallbackControl(ResourceBundle.Control.FORMAT_DEFAULT).getCandidateLocales("", locale);
    }
    
    static Locale getLookupLocale(Locale build) {
        if (build.hasExtensions() && !build.equals(JRELocaleConstants.JA_JP_JP) && !build.equals(JRELocaleConstants.TH_TH_TH)) {
            Locale.Builder builder = new Locale.Builder();
            try {
                builder.setLocale(build);
                builder.clearExtensions();
                build = builder.build();
            }
            catch (IllformedLocaleException ex) {
                config(LocaleServiceProviderPool.class, "A locale(" + build + ") has non-empty extensions, but has illformed fields.");
                build = new Locale(build.getLanguage(), build.getCountry(), build.getVariant());
            }
        }
        return build;
    }
    
    static Set access$000(LocaleServiceProviderPool localeServiceProviderPool) {
        return localeServiceProviderPool.getAvailableLocaleSet();
    }
    
    static {
        LocaleServiceProviderPool.poolOfPools = new ConcurrentHashMap();
        spiClasses = (Class[])new Class[] { BreakIteratorProvider.class, CollatorProvider.class, DateFormatProvider.class, DateFormatSymbolsProvider.class, DecimalFormatSymbolsProvider.class, NumberFormatProvider.class, CurrencyNameProvider.class, LocaleNameProvider.class, TimeZoneNameProvider.class, CalendarDataProvider.class };
        LocaleServiceProviderPool.NULL_LIST = Collections.emptyList();
    }
    
    private static class AllAvailableLocales
    {
        static final Locale[] allAvailableLocales;
        
        static {
            HashSet<Object> set = new HashSet();
            Class[] spiClasses = LocaleServiceProviderPool.spiClasses;
            for (int length = spiClasses.length, i = 0; i < length; ++i) {
                set.addAll(LocaleServiceProviderPool.access$000(LocaleServiceProviderPool.getPool(spiClasses[i])));
            }
            allAvailableLocales = (Locale[])set.toArray(new Locale[0]);
        }
    }
    
    public interface LocalizedObjectGetter<P extends LocaleServiceProvider, S>
    {
        public abstract S getObject(P p0, Locale p1, String p2, Object... p3);
    }
}
